<?php

namespace Restaurant\Phone\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Restaurant\Phone\Http\Requests\PhoneRequest;
use Restaurant\Phone\Interfaces\PhoneRepositoryInterface;
use Restaurant\Phone\Models\Phone;

/**
 * Resource controller class for phone.
 */
class PhoneResourceController extends BaseController
{

    /**
     * Initialize phone resource controller.
     *
     * @param type PhoneRepositoryInterface $phone
     *
     * @return null
     */
    public function __construct(PhoneRepositoryInterface $phone)
    {
        parent::__construct();
        $this->repository = $phone;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Restaurant\Phone\Repositories\Criteria\PhoneResourceCriteria::class);
    }

    /**
     * Display a list of phone.
     *
     * @return Response
     */
    public function index(PhoneRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Restaurant\Phone\Repositories\Presenter\PhonePresenter::class)
                ->$function();
        }

        $phones = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('phone::phone.names'))
            ->view('phone::phone.index', true)
            ->data(compact('phones', 'view'))
            ->output();
    }

    /**
     * Display phone.
     *
     * @param Request $request
     * @param Model   $phone
     *
     * @return Response
     */
    public function show(PhoneRequest $request, Phone $phone)
    {

        if ($phone->exists) {
            $view = 'phone::phone.show';
        } else {
            $view = 'phone::phone.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('phone::phone.name'))
            ->data(compact('phone'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new phone.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(PhoneRequest $request)
    {

        $phone = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('phone::phone.name')) 
            ->view('phone::phone.create', true) 
            ->data(compact('phone'))
            ->output();
    }

    /**
     * Create new phone.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(PhoneRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $phone                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('phone::phone.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('phone/phone/' . $phone->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/phone/phone'))
                ->redirect();
        }

    }

    /**
     * Show phone for editing.
     *
     * @param Request $request
     * @param Model   $phone
     *
     * @return Response
     */
    public function edit(PhoneRequest $request, Phone $phone)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('phone::phone.name'))
            ->view('phone::phone.edit', true)
            ->data(compact('phone'))
            ->output();
    }

    /**
     * Update the phone.
     *
     * @param Request $request
     * @param Model   $phone
     *
     * @return Response
     */
    public function update(PhoneRequest $request, Phone $phone)
    {
        try {
            $attributes = $request->all();

            $phone->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('phone::phone.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('phone/phone/' . $phone->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('phone/phone/' . $phone->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the phone.
     *
     * @param Model   $phone
     *
     * @return Response
     */
    public function destroy(PhoneRequest $request, Phone $phone)
    {
        try {

            $phone->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('phone::phone.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('phone/phone/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('phone/phone/' . $phone->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple phone.
     *
     * @param Model   $phone
     *
     * @return Response
     */
    public function delete(PhoneRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('phone::phone.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('phone/phone'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/phone/phone'))
                ->redirect();
        }

    }

    /**
     * Restore deleted phones.
     *
     * @param Model   $phone
     *
     * @return Response
     */
    public function restore(PhoneRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('phone::phone.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/phone/phone'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/phone/phone/'))
                ->redirect();
        }

    }

}
